Homework assignment no. 2, RPG Character generator
====================================

**Publication date:** 15th May 

**Submission deadline:** 5th June 23:59

General information
-------------------
In this assignment you will create a command line application which can be used to generate an HTML overview of a character for a very simple tabletop RPG system.

First and foremost... "What the hell is a tabletop RPG?"
It's a role playing game in which participants describe their character and act as them through speech, writings, and drawings. The entire game is usually driven by another participant (called Dungeon Master -- DM) who acts as a storyteller. 
You might have encountered the most famous tabletop RPG game system  [DnD (Dungeons & Dragons)](https://dnd.wizards.com/basics-play) or its czech counterpart [Dračí Doupě (DrD)](https://www.drdplus.info/)
  

### Game System Information
Our game system will be much simpler than those of DnD or DrD. We wont have any classes or leveling system. There will be just characters with predetermined set of skills and attributes.  
Each character has some basic information such as *name*, *age*, *occupation*. 
Additionally, each character also has a set of attributes (e.g. strength, dexterity) and set of skills (e.g.  leadership, intimidation) -- each of these has a rank between 0 and 5. 
As you play the game you will encounter many situations. Lets say your group captured certain street thug and needs to know what he did last night. 
Since your character has the interrogation skill, you decide to press on the thug and extract the information.  The dungeon master determines that you need at least 3 success points to convince the thug to spill the beans.
Since you character has 4 points in the interrogation skill, you roll with four 10-sided dice. If you get a result higher than 4 on at least 3 ouf these dice, you will succeed in interrogating the thug. 
It's a very simple, yet versatile and fast-paced system. 

### External Libraries
In this assignment you will encounter two useful java libraries

#### JCommander
JCommander is a small library for parsing command line arguments. The documentation and examples can be found [here](https://jcommander.org/).
You will have to learn just the basics by replicating what is already present in the skeleton of ```Application``` class or by looking into the docs. 

### Thymeleaf
Thymeleaf is a complex templating library which allows for templating of various documents from simple text, through html, to javascript files. 
In this homework you will be required to fill in a small part of a template. Other than that, you are encouraged to have a look at how to use this library. 
The documentation can be found [here](https://www.thymeleaf.org/doc/tutorials/3.0/usingthymeleaf.html#introducing-thymeleaf). 
     
    
### Evaluation
Beside functional correctness this assignment is focused on clean implementation and your ability to create a more complex java application.
 
The maximum number of points for this assignment is **11**.

- **4 points** for passing tests (this time attached tests are very basic).
- **4 points** for correctly working implementation.
- **3 points** for clean implementation and creativity.

Note that all this is at your seminar teacher's discretion.

### Preconditions
To successfully implement this assignment you need to learn the following

1. Working with files (java.nio package)
2. Understand more complex code provided by 3rd party 
3. Working with external libraries

### Project structure
The structure of project provided as a base for your implementation should meet the following criteria.

1. Package ```cz.muni.fi.pb162.hw03``` contains classes and interfaces provided as part of the assignment.
  - **Do not modify or add any classes or subpackages into this package.**
2. Package  ```cz.muni.fi.pb162.hw03.impl``` should contain your implementation.
  - **Anything outside this package will be ignored during evaluation.**

### Names in this document
Unless fully classified name is provided, all class names are relative to  package ```cz.muni.fi.pb162.hw03``` or ```cz.muni.fi.pb162.hw03.impl``` for classes implemented as part of your solution.

### Compiling the project
The project can be compiled and packaged in the same way you already know 

```bash
$ mvn clean compile
```

The only difference is, that unlike with seminar project, this time checks for missing documentation and style violation will produce an error. 
You can temporarily disable this behavior when running this command. 

```bash
$ mvn clean compile -Dcheckstyle.fail=false
```

You can consult your seminar teacher to help you set the ```checkstyle.fail``` property in your IDE (or just google it). 

### Submitting the assignment
The procedure to submit your solution may differ based on your seminar group. However generally it should be ok to submit ```target/homework01-2020-1.0-SNAPSHOT-sources.jar``` to the homework vault.

### Application Usage and Overview
As already mentioned, the goal is to implement a command line utility, which will produce a character sheet for our tabletop RPG.  The program will recognize the following command line options

| Name | Default Value | restriction |Description |
|------|---------------|-------------|------------|
| --name | - | - | Character's name |
| --age | - | positive integer |Character's age |
| --occupation | Citizen |- | Character's occupation |
| --health | - | 1-10 |Character's health |
| --sanity | - |1-10 | Character's sanity |
| --inventory | 42 | positive, x % 3 = 0 | Character's inventory size |
| --attrs | attributes.txt | - | Attribute file descriptor |
| --skills | skills.txt | - | Skills file descriptor |
| --out | character.html | - | output file name |
| --encoding | UTF-8 | - | Encoding of input files |
| --help |false | - | when present displays program help|

Parameters without default value are mandatory.
 
An example use of the program could look like this one bellow.

```bash
java -jar rpg-gen.jar --name "Stephen Mercer" --age 32 --occupation "Investigative Journalist" --health 5 --sanity 5  --inventory=45 \
                      --attrs "stephen_attrs.txt" --skills "stephen_skills.txt" 
```

Example of the file descriptors can be found in ```src/test/resources```. 
In general the format is: 

```text
AbilityOrSkill 5
AbilityOrSkill R
```

Each ability or sill is placed on single line defined as combination of name and level. 
Level can be specified either as a number between 0 and 5 or as "R". When "R" is used as level, then the actual value is computed as a random within mentioned range.

###  Step 1: Implementing Character class
Implement the ```Character``` class with the following attributes. In order to help you a bit there is already a very sparse skeleton in place. 

| Name | Type | Description |
| -----|------|-------------|
| name | String | character's name |
| age  | int    | characters age |
| occupation | String | character's occupation |
| health | int  | character's health |
| sanity | int | character's sanity |
| attributes | List<SkillOrAttribute> | character's attributes |
| skills | List<SkillOrAttribute> | character's skills |

Since the class has quite a few attributes, you should provide ```Builder``` class -- a  builder implementation which will be used to construct instances of ``Character`` class.
This builder should implement ```Buildable``` interface as well as ```read.AttributeReader``` and ```read.SkillReader```.

### Step 2: Implementing the command line interface
Complete the implementation of ``Application`` class in order for it to recognise all of the command line parameters described above.
In order to validate some of the values provided on command line, you will need to read section *4.1. Individual parameter validation* in JCommander's documentation.

Program usage info should be displayed when ``--help`` is specified as command line parameter.

### Character to HTML file
Running the ``main`` method of ``Application`` class should produce a HTML file with character information. 
To achieve this you should use the Thymeleaf library already mentioned in this assignment. 
To have easier time working with this library, you can use the class ```impl.AbstractHtmlWriter``` and prepared template located in ```src/main/resources```.
However this template is not complete and you will have to fill in few things.

Pleas notice that ``AbstractHtmlWriter`` is located inside the ```impl``` package, which means that you can modify it to suit your needs.   